@@ -22,6 +22,7 @@
 #include <gio/gio.h>
 #include <glib.h>
 #include <libupower-glib/upower.h>
+#include <gtk/gtk.h>
 
 #include "egg-debug.h"
 #include "gpm-button.h"
@@ -31,6 +32,10 @@
 #include "gpm-kbd-backlight.h"
 #include "gsd-media-keys-window.h"
 
+#if !GTK_CHECK_VERSION(3,0,0)
+#define gtk_widget_get_preferred_size(x,y,z) gtk_widget_size_request(x,y)
+#endif
+
 #define GPM_KBD_BACKLIGHT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_KBD_BACKLIGHT, GpmKbdBacklightPrivate))
 
 struct GpmKbdBacklightPrivate
@@ -185,6 +190,11 @@ gpm_kbd_backlight_dialog_show (GpmKbdBacklight *backlight)
 	GdkScreen     *pointer_screen;
 	GdkRectangle   geometry;
 	int            monitor;
+#if GTK_CHECK_VERSION(3,0,0)
+        GdkDisplay    *display;
+        GdkDeviceManager *device_manager;
+        GdkDevice     *device;
+#endif
 
 	/*
 	 * get the window size
@@ -192,7 +202,7 @@ gpm_kbd_backlight_dialog_show (GpmKbdBacklight *backlight)
 	 * know its true size, yet, so we need to jump through hoops
 	 */
 	gtk_window_get_default_size (GTK_WINDOW (backlight->priv->popup), &orig_w, &orig_h);
-	gtk_widget_size_request (backlight->priv->popup, &win_req);
+	gtk_widget_get_preferred_size (backlight->priv->popup, NULL, &win_req);
 
 	if (win_req.width > orig_w) {
 		orig_w = win_req.width;
@@ -202,11 +212,21 @@ gpm_kbd_backlight_dialog_show (GpmKbdBacklight *backlight)
 	}
 
 	pointer_screen = NULL;
+#if GTK_CHECK_VERSION(3,0,0)
+        display = gtk_widget_get_display (backlight->priv->popup);
+        device_manager = gdk_display_get_device_manager (display);
+        device = gdk_device_manager_get_client_pointer (device_manager);
+        gdk_device_get_position (device,
+				 &pointer_screen,
+				 &pointer_x,
+				 &pointer_y);
+#else
 	gdk_display_get_pointer (gtk_widget_get_display (backlight->priv->popup),
 				 &pointer_screen,
 				 &pointer_x,
 				 &pointer_y,
 				 NULL);
+#endif
 	monitor = gdk_screen_get_monitor_at_point (pointer_screen,
 						   pointer_x,
 						   pointer_y);
